@@ -140,6 +140,10 @@ template <typename Container> void GenericDNSPacketWriter<Container>::xfr48BitIn
   d_content.insert(d_content.end(), bytes, bytes + sizeof(bytes));
 }
 
+template <typename Container> void GenericDNSPacketWriter<Container>::xfrNodeOrLocatorID(const NodeOrLocatorID& val)
+{
+  d_content.insert(d_content.end(), val.content, val.content + sizeof(val.content));
+}
 
 template <typename Container> void GenericDNSPacketWriter<Container>::xfr32BitInt(uint32_t val)
 {
@@ -378,6 +382,11 @@ template <typename Container> void GenericDNSPacketWriter<Container>::xfrBlob(co
   d_content.insert(d_content.end(), ptr, ptr+blob.size());
 }
 
+template <typename Container> void GenericDNSPacketWriter<Container>::xfrBlob(const std::vector<uint8_t>& blob)
+{
+  d_content.insert(d_content.end(), blob.begin(), blob.end());
+}
+
 template <typename Container> void GenericDNSPacketWriter<Container>::xfrBlobNoSpaces(const string& blob, int  )
 {
   xfrBlob(blob);
@@ -423,19 +432,19 @@ template <typename Container> void GenericDNSPacketWriter<Container>::xfrSvcPara
       break;
     case SvcParam::ipv4hint:
       xfr16BitInt(param.getIPHints().size() * 4); // size
-      for (auto a: param.getIPHints()) {
+      for (const auto& a: param.getIPHints()) {
         xfrCAWithoutPort(param.getKey(), a);
       }
       break;
     case SvcParam::ipv6hint:
       xfr16BitInt(param.getIPHints().size() * 16); // size
-      for (auto a: param.getIPHints()) {
+      for (const auto& a: param.getIPHints()) {
         xfrCAWithoutPort(param.getKey(), a);
       }
       break;
-    case SvcParam::echconfig:
-      xfr16BitInt(param.getEchConfig().size()); // size
-      xfrBlobNoSpaces(param.getEchConfig());
+    case SvcParam::ech:
+      xfr16BitInt(param.getECH().size()); // size
+      xfrBlobNoSpaces(param.getECH());
       break;
     default:
       xfr16BitInt(param.getValue().size());
