@@ -92,7 +92,7 @@ struct args_t {
     fcontext_t prev_ctx = nullptr;
 #endif
     pdns_ucontext_t* self = nullptr;
-    boost::function<void(void)>* work = nullptr;
+    std::function<void(void)>* work = nullptr;
 };
 
 extern "C" {
@@ -212,7 +212,7 @@ pdns_swapcontext
 
 void
 pdns_makecontext
-(pdns_ucontext_t& ctx, boost::function<void(void)>& start) {
+(pdns_ucontext_t& ctx, std::function<void(void)>& start) {
     assert (ctx.uc_link);
     assert (ctx.uc_stack.size() >= 8192);
     assert (!ctx.uc_mcontext);
