@@ -5,6 +5,7 @@
 #include "rec_channel.hh"
 
 #include "logger.hh"
+#include "logging.hh"
 
 #ifdef HAVE_NET_SNMP
 
@@ -146,6 +147,25 @@ static const oid udp6InCsumErrorsOID[] = {RECURSOR_STATS_OID, 123};
 static const oid sourceDisallowedNotifyOID[] = {RECURSOR_STATS_OID, 124};
 static const oid zoneDisallowedNotifyOID[] = {RECURSOR_STATS_OID, 125};
 static const oid nonResolvingNameserverEntriesOID[] = {RECURSOR_STATS_OID, 126};
+static const oid maintenanceUSecOID[] = {RECURSOR_STATS_OID, 127};
+static const oid maintenanceCallsOID[] = {RECURSOR_STATS_OID, 128};
+
+static const oid rcode0AnswersOID[] = {RECURSOR_STATS_OID, 129};
+static const oid rcode1AnswersOID[] = {RECURSOR_STATS_OID, 130};
+static const oid rcode2AnswersOID[] = {RECURSOR_STATS_OID, 131};
+static const oid rcode3AnswersOID[] = {RECURSOR_STATS_OID, 132};
+static const oid rcode4AnswersOID[] = {RECURSOR_STATS_OID, 133};
+static const oid rcode5AnswersOID[] = {RECURSOR_STATS_OID, 134};
+static const oid rcode6AnswersOID[] = {RECURSOR_STATS_OID, 135};
+static const oid rcode7AnswersOID[] = {RECURSOR_STATS_OID, 136};
+static const oid rcode8AnswersOID[] = {RECURSOR_STATS_OID, 137};
+static const oid rcode9AnswersOID[] = {RECURSOR_STATS_OID, 138};
+static const oid rcode10AnswersOID[] = {RECURSOR_STATS_OID, 139};
+static const oid rcode11AnswersOID[] = {RECURSOR_STATS_OID, 140};
+static const oid rcode12AnswersOID[] = {RECURSOR_STATS_OID, 141};
+static const oid rcode13AnswersOID[] = {RECURSOR_STATS_OID, 142};
+static const oid rcode14AnswersOID[] = {RECURSOR_STATS_OID, 143};
+static const oid rcode15AnswersOID[] = {RECURSOR_STATS_OID, 144};
 
 static std::unordered_map<oid, std::string> s_statsMap;
 
@@ -200,12 +220,14 @@ static int handleDisabledCounter64Stats(netsnmp_mib_handler* handler,
 static void registerCounter64Stat(const std::string& name, const oid statOID[], size_t statOIDLength)
 {
   if (statOIDLength != OID_LENGTH(questionsOID)) {
-    g_log << Logger::Error << "Invalid OID for SNMP Counter64 statistic " << name << endl;
+    SLOG(g_log << Logger::Error << "Invalid OID for SNMP Counter64 statistic " << name << endl,
+         g_slog->withName("snmp")->info(Logr::Error, "Invalid OID for SNMP Counter64 statistic", "name", Logging::Loggable(name)));
     return;
   }
 
   if (s_statsMap.find(statOID[statOIDLength - 1]) != s_statsMap.end()) {
-    g_log << Logger::Error << "OID for SNMP Counter64 statistic " << name << " has already been registered" << endl;
+    SLOG(g_log << Logger::Error << "OID for SNMP Counter64 statistic " << name << " has already been registered" << endl,
+         g_slog->withName("snmp")->info(Logr::Error, "OID for SNMP Counter64 statistic has already been registered", "name", Logging::Loggable(name)));
     return;
   }
 
@@ -381,5 +403,26 @@ RecursorSNMPAgent::RecursorSNMPAgent(const std::string& name, const std::string&
   registerCounter64Stat("almost-expired-run", almostExpiredRun, OID_LENGTH(almostExpiredRun));
   registerCounter64Stat("almost-expired-exceptions", almostExpiredExceptions, OID_LENGTH(almostExpiredExceptions));
   registerCounter64Stat("non-resolving-nameserver-entries", nonResolvingNameserverEntriesOID, OID_LENGTH(nonResolvingNameserverEntriesOID));
+  registerCounter64Stat("maintenance-usec", maintenanceUSecOID, OID_LENGTH(maintenanceUSecOID));
+  registerCounter64Stat("maintenance-calls", maintenanceCallsOID, OID_LENGTH(maintenanceCallsOID));
+
+#define RCODE(num) registerCounter64Stat("auth-" + RCode::to_short_s(num) + "-answers", rcode##num##AnswersOID, OID_LENGTH(rcode##num##AnswersOID))
+  RCODE(0);
+  RCODE(1);
+  RCODE(2);
+  RCODE(3);
+  RCODE(4);
+  RCODE(5);
+  RCODE(6);
+  RCODE(7);
+  RCODE(8);
+  RCODE(9);
+  RCODE(10);
+  RCODE(11);
+  RCODE(12);
+  RCODE(13);
+  RCODE(14);
+  RCODE(15);
+
 #endif /* HAVE_NET_SNMP */
 }
