@@ -114,14 +114,15 @@ static struct taskstats s_resolve_tasks;
 
 static void resolve(const struct timeval& now, bool logErrors, const pdns::ResolveTask& task) noexcept
 {
-  auto log = g_slog->withName("taskq")->withValues("name", Logging::Loggable(task.d_qname), "qtype", Logging::Loggable(QType(task.d_qtype).toString()));
+  auto log = g_slog->withName("taskq")->withValues("name", Logging::Loggable(task.d_qname), "qtype", Logging::Loggable(QType(task.d_qtype).toString()), "netmask", Logging::Loggable(task.d_netmask.empty() ? "" : task.d_netmask.toString()));
   const string msg = "Exception while running a background ResolveTask";
   SyncRes sr(now);
   vector<DNSRecord> ret;
   sr.setRefreshAlmostExpired(task.d_refreshMode);
+  sr.setQuerySource(task.d_netmask);
   bool ex = true;
   try {
-    log->info(Logr::Debug, "resolving");
+    log->info(Logr::Debug, "resolving", "refresh", Logging::Loggable(task.d_refreshMode));
     int res = sr.beginResolve(task.d_qname, QType(task.d_qtype), QClass::IN, ret);
     ex = false;
     log->info(Logr::Debug, "done", "rcode", Logging::Loggable(res), "records", Logging::Loggable(ret.size()));
@@ -172,10 +173,10 @@ static void tryDoT(const struct timeval& now, bool logErrors, const pdns::Resolv
   sr.setRefreshAlmostExpired(false);
   bool ex = true;
   try {
-    log->info(Logr::Warning, "trying DoT");
+    log->info(Logr::Debug, "trying DoT");
     bool ok = sr.tryDoT(task.d_qname, QType(task.d_qtype), task.d_nsname, task.d_ip, now.tv_sec);
     ex = false;
-    log->info(Logr::Warning, "done", "ok", Logging::Loggable(ok));
+    log->info(Logr::Debug, "done", "ok", Logging::Loggable(ok));
   }
   catch (const std::exception& e) {
     log->error(Logr::Error, msg, e.what());
@@ -231,14 +232,14 @@ bool runTaskOnce(bool logErrors)
   return true;
 }
 
-void pushAlmostExpiredTask(const DNSName& qname, uint16_t qtype, time_t deadline)
+void pushAlmostExpiredTask(const DNSName& qname, uint16_t qtype, time_t deadline, const Netmask& netmask)
 {
   if (SyncRes::isUnsupported(qtype)) {
-    auto log = g_slog->withName("taskq")->withValues("name", Logging::Loggable(qname), "qtype", Logging::Loggable(QType(qtype).toString()));
+    auto log = g_slog->withName("taskq")->withValues("name", Logging::Loggable(qname), "qtype", Logging::Loggable(QType(qtype).toString()), "netmask", Logging::Loggable(netmask.empty() ? "" : netmask.toString()));
     log->error(Logr::Error, "Cannot push task", "qtype unsupported");
     return;
   }
-  pdns::ResolveTask task{qname, qtype, deadline, true, resolve, {}, {}};
+  pdns::ResolveTask task{qname, qtype, deadline, true, resolve, {}, {}, netmask};
   if (s_taskQueue.lock()->queue.push(std::move(task))) {
     ++s_almost_expired_tasks.pushed;
   }
@@ -251,7 +252,7 @@ void pushResolveTask(const DNSName& qname, uint16_t qtype, time_t now, time_t de
     log->error(Logr::Error, "Cannot push task", "qtype unsupported");
     return;
   }
-  pdns::ResolveTask task{qname, qtype, deadline, false, resolve, {}, {}};
+  pdns::ResolveTask task{qname, qtype, deadline, false, resolve, {}, {}, {}};
   auto lock = s_taskQueue.lock();
   bool inserted = lock->rateLimitSet.insert(now, task);
   if (inserted) {
@@ -269,7 +270,7 @@ bool pushTryDoTTask(const DNSName& qname, uint16_t qtype, const ComboAddress& ip
     return false;
   }
 
-  pdns::ResolveTask task{qname, qtype, deadline, false, tryDoT, ip, nsname};
+  pdns::ResolveTask task{qname, qtype, deadline, false, tryDoT, ip, nsname, {}};
   bool pushed = s_taskQueue.lock()->queue.push(std::move(task));
   if (pushed) {
     ++s_almost_expired_tasks.pushed;
