@@ -11,6 +11,7 @@
 #else
 #include "dolog.hh"
 #endif
+#include "logging.hh"
 
 bool CircularWriteBuffer::hasRoomFor(const std::string& str) const
 {
@@ -94,6 +95,19 @@ bool CircularWriteBuffer::flush(int fd)
   return true;
 }
 
+const std::string& RemoteLoggerInterface::toErrorString(Result r)
+{
+  static const std::array<std::string,5> str = {
+    "Queued",
+    "Queue full, dropping",
+    "Not sending too large protobuf message",
+    "Submiting to queue failed",
+    "?"
+  };
+  auto i = static_cast<unsigned int>(r);
+  return str[std::min(i, 4U)];
+}
+
 RemoteLogger::RemoteLogger(const ComboAddress& remote, uint16_t timeout, uint64_t maxQueuedBytes, uint8_t reconnectWaitTime, bool asyncConnect): d_remote(remote), d_timeout(timeout), d_reconnectWaitTime(reconnectWaitTime), d_asyncConnect(asyncConnect), d_runtime({CircularWriteBuffer(maxQueuedBytes), nullptr})
 {
   if (!d_asyncConnect) {
@@ -119,7 +133,8 @@ bool RemoteLogger::reconnect()
   }
   catch (const std::exception& e) {
 #ifdef WE_ARE_RECURSOR
-    g_log<<Logger::Warning<<"Error connecting to remote logger "<<d_remote.toStringWithPort()<<": "<<e.what()<<std::endl;
+    SLOG(g_log<<Logger::Warning<<"Error connecting to remote logger "<<d_remote.toStringWithPort()<<": "<<e.what()<<std::endl,
+         g_slog->withName("protobuf")->error(Logr::Error, e.what(), "Exception while connection to remote logger", "address", Logging::Loggable(d_remote)));
 #else
     warnlog("Error connecting to remote logger %s: %s", d_remote.toStringWithPort(), e.what());
 #endif
@@ -129,51 +144,47 @@ bool RemoteLogger::reconnect()
   return true;
 }
 
-void RemoteLogger::queueData(const std::string& data)
+RemoteLoggerInterface::Result RemoteLogger::queueData(const std::string& data)
 {
-  if (data.size() > std::numeric_limits<uint16_t>::max()) {
-    const auto msg = "Not sending too large protobuf message";
-#ifdef WE_ARE_RECURSOR
-    g_log<<Logger::Info<<msg<<endl;
-#else
-    warnlog(msg);
-#endif
-    return;
-  }
-
   auto runtime = d_runtime.lock();
 
+  if (data.size() > std::numeric_limits<uint16_t>::max()) {
+    ++runtime->d_stats.d_tooLarge;
+    return Result::TooLarge;
+  }
+
   if (!runtime->d_writer.hasRoomFor(data)) {
     /* not connected, queue is full, just drop */
     if (!runtime->d_socket) {
-      ++d_drops;
-      return;
+      ++runtime->d_stats.d_pipeFull;
+      return Result::PipeFull;
     }
     try {
       /* we try to flush some data */
       if (!runtime->d_writer.flush(runtime->d_socket->getHandle())) {
         /* but failed, let's just drop */
-        ++d_drops;
-        return;
+        ++runtime->d_stats.d_pipeFull;
+        return Result::PipeFull;
       }
 
       /* see if we freed enough data */
       if (!runtime->d_writer.hasRoomFor(data)) {
         /* we didn't */
-        ++d_drops;
-        return;
+        ++runtime->d_stats.d_pipeFull;
+        return Result::PipeFull;
       }
     }
     catch(const std::exception& e) {
       //      cout << "Got exception writing: "<<e.what()<<endl;
-      ++d_drops;
       runtime->d_socket.reset();
-      return;
+      ++runtime->d_stats.d_otherError;
+      return Result::OtherError;
     }
   }
 
   runtime->d_writer.write(data);
-  ++d_processed;
+  ++runtime->d_stats.d_queued;
+  return Result::Queued;
 }
 
 void RemoteLogger::maintenanceThread() 
@@ -228,10 +239,12 @@ void RemoteLogger::maintenanceThread()
   }
   catch (const std::exception& e)
   {
-    cerr << "Remote Logger's maintenance thead died on: " << e.what() << endl;
+    SLOG(cerr << "Remote Logger's maintenance thread died on: " << e.what() << endl,
+         g_slog->withName("protobuf")->error(Logr::Error, e.what(), "Remote Logger's maintenance thread died"));
   }
   catch (...) {
-    cerr << "Remote Logger's maintenance thead died on unknown exception" << endl;
+    SLOG(cerr << "Remote Logger's maintenance thread died on unknown exception" << endl,
+         g_slog->withName("protobuf")->info(Logr::Error, "Remote Logger's maintenance thread died"));
   }
 }
 
@@ -241,3 +254,4 @@ RemoteLogger::~RemoteLogger()
 
   d_thread.join();
 }
+
