@@ -22,30 +22,20 @@
 
 #include "taskqueue.hh"
 
-#include "logger.hh"
+#include "logging.hh"
 #include "syncres.hh"
 
 namespace pdns
 {
 
-bool TaskQueue::empty() const
-{
-  return d_queue.empty();
-}
-
-size_t TaskQueue::size() const
-{
-  return d_queue.size();
-}
-
-void TaskQueue::push(ResolveTask&& task)
+bool TaskQueue::push(ResolveTask&& task)
 {
   // Insertion fails if it's already there, no problem since we're already scheduled
-  // and the deadline would remain the same anyway.
-  auto result = d_queue.insert(std::move(task));
-  if (result.second) {
+  auto result = d_queue.insert(std::move(task)).second;
+  if (result) {
     d_pushes++;
   }
+  return result;
 }
 
 ResolveTask TaskQueue::pop()
@@ -55,49 +45,33 @@ ResolveTask TaskQueue::pop()
   return ret;
 }
 
-bool TaskQueue::runOnce(bool logErrors)
+bool ResolveTask::run(bool logErrors)
 {
-  if (d_queue.empty()) {
+  if (d_func == nullptr) {
+    auto log = g_slog->withName("taskq")->withValues("name", Logging::Loggable(d_qname), "qtype", Logging::Loggable(QType(d_qtype).toString()));
+    log->error(Logr::Debug, "null task");
     return false;
   }
-  ResolveTask task = pop();
-  if (task.func == nullptr) {
-    g_log << Logger::Debug << "TaskQueue: null task for " << task.d_qname.toString() << '|' << QType(task.d_qtype).toString() << endl;
-    return true;
-  }
   struct timeval now;
   Utility::gettimeofday(&now);
-  if (task.d_deadline >= now.tv_sec) {
-    task.func(now, logErrors, task);
+  if (d_deadline >= now.tv_sec) {
+    d_func(now, logErrors, *this);
   }
   else {
     // Deadline passed
-    g_log << Logger::Debug << "TaskQueue: deadline for " << task.d_qname.toString() << '|' << QType(task.d_qtype).toString() << " passed" << endl;
-    d_expired++;
+    auto log = g_slog->withName("taskq")->withValues("name", Logging::Loggable(d_qname), "qtype", Logging::Loggable(QType(d_qtype).toString()));
+    log->info(Logr::Debug, "deadline passed");
+    return true;
   }
-  return true;
-}
-
-void TaskQueue::runAll(bool logErrors)
-{
-  while (runOnce(logErrors)) {
-    /* empty */
-  }
-}
-
-uint64_t* TaskQueue::getPushes() const
-{
-  return new uint64_t(d_pushes);
-}
-
-uint64_t* TaskQueue::getExpired() const
-{
-  return new uint64_t(d_expired);
-}
-
-uint64_t* TaskQueue::getSize() const
-{
-  return new uint64_t(size());
+  return false;
 }
 
 } /* namespace pdns */
+
+namespace boost
+{
+size_t hash_value(const ComboAddress& a)
+{
+  return ComboAddress::addressOnlyHash()(a);
+}
+}
