@@ -5,16 +5,12 @@
 #endif
 
 #include "rpzloader.hh"
+#include "syncres.hh"
+
 #include <boost/test/unit_test.hpp>
 
 // Provide stubs for some symbols
 bool g_logRPZChanges{false};
-ComboAddress getQueryLocalAddress(int family, uint16_t port)
-{
-  cerr << "getQueryLocalAddress() STUBBED IN TEST!" << endl;
-  BOOST_ASSERT(false);
-  return ComboAddress();
-}
 
 BOOST_AUTO_TEST_SUITE(rpzloader_cc)
 
@@ -23,10 +19,10 @@ BOOST_AUTO_TEST_CASE(test_rpz_loader)
 
   string tests[][2] = {
     {"32.3.2.168.192", "192.168.2.3/32"},
-    {"27.73.2.168.192", "192.168.2.73/27"},
+    {"27.73.2.168.192", "192.168.2.64/27"},
     {"24.0.2.168.192", "192.168.2.0/24"},
     {"128.57.zz.1.0.db8.2001", "2001:db8:0:1::57/128"},
-    {"48.zz.1.0.db8.2001", "2001:db8:0:1::/48"},
+    {"48.zz.1.0.db8.2001", "2001:db8::/48"},
     {"128.5.C0A8.FFFF.0.1.0.db8.2001", "2001:db8:0:1:0:ffff:c0a8:5/128"},
 
     {"21.0.248.44.5", "5.44.248.0/21"},
